@@ -1,16 +1,28 @@
-use std::thread::JoinHandle;
+use std::{sync::Arc, thread::JoinHandle};
 
 use clap::ArgMatches;
 use config::{Config, ConfigError, Environment, File};
 use log::debug;
-use oura::pipelining::{
-    BootstrapResult, FilterProvider, PartialBootstrapResult, SinkProvider, SourceProvider,
-    StageReceiver,
+use serde::Deserialize;
+
+use oura::{
+    pipelining::{
+        BootstrapResult, FilterProvider, PartialBootstrapResult, SinkProvider, SourceProvider,
+        StageReceiver,
+    },
+    utils::{cursor, ChainWellKnownInfo, Utils, WithUtils},
+    Error,
 };
+
+use oura::filters::noop::Config as NoopFilterConfig;
+use oura::filters::selection::Config as SelectionConfig;
+use oura::sinks::stdout::Config as StdoutConfig;
 use oura::sinks::terminal::Config as TerminalConfig;
 use oura::sources::n2c::Config as N2CConfig;
 use oura::sources::n2n::Config as N2NConfig;
-use serde_derive::Deserialize;
+
+#[cfg(feature = "logs")]
+use oura::sinks::logs::Config as WriterConfig;
 
 #[cfg(feature = "webhook")]
 use oura::sinks::webhook::Config as WebhookConfig;
@@ -21,13 +33,8 @@ use oura::sinks::kafka::Config as KafkaConfig;
 #[cfg(feature = "elasticsink")]
 use oura::sinks::elastic::Config as ElasticConfig;
 
-use oura::filters::noop::Config as NoopFilterConfig;
-use oura::filters::selection::Config as SelectionConfig;
-
 #[cfg(feature = "fingerprint")]
-use oura::filters::fingerpint::Config as FingerprintConfig;
-
-use crate::Error;
+use oura::filters::fingerprint::Config as FingerprintConfig;
 
 #[derive(Debug, Deserialize)]
 #[serde(tag = "type")]
@@ -36,12 +43,10 @@ enum Source {
     N2N(N2NConfig),
 }
 
-impl SourceProvider for Source {
-    fn bootstrap(&self) -> PartialBootstrapResult {
-        match self {
-            Source::N2C(c) => c.bootstrap(),
-            Source::N2N(c) => c.bootstrap(),
-        }
+fn bootstrap_source(config: Source, utils: Arc<Utils>) -> PartialBootstrapResult {
+    match config {
+        Source::N2C(config) => WithUtils::new(config, utils).bootstrap(),
+        Source::N2N(config) => WithUtils::new(config, utils).bootstrap(),
     }
 }
 
@@ -71,6 +76,10 @@ impl FilterProvider for Filter {
 #[serde(tag = "type")]
 enum Sink {
     Terminal(TerminalConfig),
+    Stdout(StdoutConfig),
+
+    #[cfg(feature = "logs")]
+    Logs(WriterConfig),
 
     #[cfg(feature = "webhook")]
     Webhook(WebhookConfig),
@@ -82,20 +91,22 @@ enum Sink {
     Elastic(ElasticConfig),
 }
 
-impl SinkProvider for Sink {
-    fn bootstrap(&self, input: StageReceiver) -> BootstrapResult {
-        match self {
-            Sink::Terminal(c) => c.bootstrap(input),
+fn bootstrap_sink(config: Sink, input: StageReceiver, utils: Arc<Utils>) -> BootstrapResult {
+    match config {
+        Sink::Terminal(c) => WithUtils::new(c, utils).bootstrap(input),
+        Sink::Stdout(c) => WithUtils::new(c, utils).bootstrap(input),
 
-            #[cfg(feature = "webhook")]
-            Sink::Webhook(c) => c.bootstrap(input),
+        #[cfg(feature = "logs")]
+        Sink::Logs(c) => WithUtils::new(c, utils).bootstrap(input),
 
-            #[cfg(feature = "kafkasink")]
-            Sink::Kafka(c) => c.bootstrap(input),
+        #[cfg(feature = "webhook")]
+        Sink::Webhook(c) => WithUtils::new(c, utils).bootstrap(input),
 
-            #[cfg(feature = "elasticsink")]
-            Sink::Elastic(c) => c.bootstrap(input),
-        }
+        #[cfg(feature = "kafkasink")]
+        Sink::Kafka(c) => WithUtils::new(c, utils).bootstrap(input),
+
+        #[cfg(feature = "elasticsink")]
+        Sink::Elastic(c) => WithUtils::new(c, utils).bootstrap(input),
     }
 }
 
@@ -107,6 +118,10 @@ struct ConfigRoot {
     filters: Vec<Filter>,
 
     sink: Sink,
+
+    chain: Option<ChainWellKnownInfo>,
+
+    cursor: Option<cursor::Config>,
 }
 
 impl ConfigRoot {
@@ -131,22 +146,40 @@ impl ConfigRoot {
     }
 }
 
+fn bootstrap_utils(chain: Option<ChainWellKnownInfo>, cursor: Option<cursor::Config>) -> Utils {
+    let well_known = chain.unwrap_or_default();
+
+    let cursor = cursor.map(cursor::Provider::initialize);
+
+    Utils::new(well_known, cursor)
+}
+
 /// Sets up the whole pipeline from configuration
-fn bootstrap(config: &ConfigRoot) -> Result<Vec<JoinHandle<()>>, Error> {
+fn bootstrap(config: ConfigRoot) -> Result<Vec<JoinHandle<()>>, Error> {
+    let ConfigRoot {
+        source,
+        filters,
+        sink,
+        chain,
+        cursor,
+    } = config;
+
+    let utils = Arc::new(bootstrap_utils(chain, cursor));
+
     let mut threads = Vec::with_capacity(10);
 
-    let (source_handle, source_rx) = config.source.bootstrap()?;
+    let (source_handle, source_rx) = bootstrap_source(source, utils.clone())?;
     threads.push(source_handle);
 
     let mut last_rx = source_rx;
 
-    for filter in config.filters.iter() {
+    for filter in filters.iter() {
         let (filter_handle, filter_rx) = filter.bootstrap(last_rx)?;
         threads.push(filter_handle);
         last_rx = filter_rx;
     }
 
-    let sink_handle = config.sink.bootstrap(last_rx)?;
+    let sink_handle = bootstrap_sink(sink, last_rx, utils)?;
     threads.push(sink_handle);
 
     Ok(threads)
@@ -164,7 +197,7 @@ pub fn run(args: &ArgMatches) -> Result<(), Error> {
 
     debug!("daemon starting with this config: {:?}", root);
 
-    let threads = bootstrap(&root)?;
+    let threads = bootstrap(root)?;
 
     // TODO: refactor into new loop that monitors thread health
     for handle in threads {
