@@ -1,13 +1,14 @@
-use std::str::FromStr;
+use std::{str::FromStr, sync::Arc};
 
 use clap::ArgMatches;
 use oura::{
     mapper::Config as MapperConfig,
-    pipelining::{PartialBootstrapResult, SinkProvider, SourceProvider},
-    sources::{AddressArg, BearerKind},
+    pipelining::{SinkProvider, SourceProvider},
+    sources::{AddressArg, BearerKind, MagicArg},
+    utils::{ChainWellKnownInfo, Utils, WithUtils},
 };
 
-use serde_derive::Deserialize;
+use serde::Deserialize;
 
 use oura::sources::n2c::Config as N2CConfig;
 use oura::sources::n2n::Config as N2NConfig;
@@ -37,15 +38,6 @@ enum WatchSource {
     N2N(N2NConfig),
 }
 
-impl SourceProvider for WatchSource {
-    fn bootstrap(&self) -> PartialBootstrapResult {
-        match self {
-            WatchSource::N2C(c) => c.bootstrap(),
-            WatchSource::N2N(c) => c.bootstrap(),
-        }
-    }
-}
-
 pub fn run(args: &ArgMatches) -> Result<(), Error> {
     env_logger::builder()
         .filter_level(log::LevelFilter::Error)
@@ -62,8 +54,8 @@ pub fn run(args: &ArgMatches) -> Result<(), Error> {
     };
 
     let magic = match args.is_present("magic") {
-        true => Some(args.value_of_t("magic")?),
-        false => None,
+        true => args.value_of_t("magic")?,
+        false => MagicArg::default(),
     };
 
     let since = match args.is_present("since") {
@@ -80,32 +72,48 @@ pub fn run(args: &ArgMatches) -> Result<(), Error> {
         (false, BearerKind::Unix) => PeerMode::AsClient,
     };
 
+    let throttle = match args.is_present("throttle") {
+        true => Some(args.value_of_t("throttle")?),
+        false => None,
+    };
+
     let mapper = MapperConfig {
         include_block_end_events: true,
         ..Default::default()
     };
 
+    let well_known = ChainWellKnownInfo::try_from_magic(*magic)?;
+
+    let utils = Arc::new(Utils::new(well_known, None));
+
+    #[allow(deprecated)]
     let source_setup = match mode {
         PeerMode::AsNode => WatchSource::N2N(N2NConfig {
             address: AddressArg(bearer, socket),
-            magic,
+            magic: Some(magic),
             well_known: None,
             mapper,
             since,
         }),
         PeerMode::AsClient => WatchSource::N2C(N2CConfig {
             address: AddressArg(bearer, socket),
-            magic,
+            magic: Some(magic),
             well_known: None,
             mapper,
             since,
         }),
     };
 
-    let sink_setup = oura::sinks::terminal::Config::default();
+    let sink_setup = oura::sinks::terminal::Config {
+        throttle_min_span_millis: throttle,
+    };
 
-    let (source_handle, source_output) = source_setup.bootstrap()?;
-    let sink_handle = sink_setup.bootstrap(source_output)?;
+    let (source_handle, source_output) = match source_setup {
+        WatchSource::N2C(c) => WithUtils::new(c, utils.clone()).bootstrap()?,
+        WatchSource::N2N(c) => WithUtils::new(c, utils.clone()).bootstrap()?,
+    };
+
+    let sink_handle = WithUtils::new(sink_setup, utils).bootstrap(source_output)?;
 
     sink_handle.join().map_err(|_| "error in sink thread")?;
     source_handle.join().map_err(|_| "error in source thread")?;
