@@ -1,22 +1,21 @@
 use std::collections::HashMap;
 
 use minicbor::bytes::ByteVec;
+use pallas::crypto::hash::Hash;
 use pallas::ledger::alonzo::{
     self as alonzo, AuxiliaryData, Block, Certificate, InstantaneousRewardSource,
     InstantaneousRewardTarget, Metadatum, Relay, TransactionInput, TransactionOutput, Value,
 };
 use pallas::ledger::alonzo::{NetworkId, TransactionBody, TransactionBodyComponent};
 
-use bech32::{self, ToBase32};
-
 use serde_json::{json, Value as JsonValue};
 
-use crate::framework::{
+use crate::model::{
     BlockRecord, EventData, MetadataRecord, MetadatumRendition, MintRecord, OutputAssetRecord,
     StakeCredential, TransactionRecord, TxInputRecord, TxOutputRecord,
 };
 
-use crate::framework::Error;
+use crate::Error;
 
 use super::EventWriter;
 
@@ -30,6 +29,12 @@ impl ToHex for Vec<u8> {
     }
 }
 
+impl<const BYTES: usize> ToHex for Hash<BYTES> {
+    fn to_hex(&self) -> String {
+        hex::encode(self)
+    }
+}
+
 impl From<&alonzo::StakeCredential> for StakeCredential {
     fn from(other: &alonzo::StakeCredential) -> Self {
         match other {
@@ -39,17 +44,6 @@ impl From<&alonzo::StakeCredential> for StakeCredential {
     }
 }
 
-pub trait ToBech32 {
-    fn try_to_bech32(&self, hrp: &str) -> Result<String, Error>;
-}
-
-impl ToBech32 for Vec<u8> {
-    fn try_to_bech32(&self, hrp: &str) -> Result<String, Error> {
-        let enc = bech32::encode(hrp, self.to_base32(), bech32::Variant::Bech32)?;
-        Ok(enc)
-    }
-}
-
 fn ip_string_from_bytes(bytes: &[u8]) -> String {
     format!("{}.{}.{}.{}", bytes[0], bytes[1], bytes[2], bytes[3])
 }
@@ -157,8 +151,10 @@ impl EventWriter {
         output: &TransactionOutput,
     ) -> Result<TxOutputRecord, Error> {
         Ok(TxOutputRecord {
-            // TODO: add correct bech32 prefix depending on network
-            address: output.address.try_to_bech32("addr")?,
+            address: self
+                .utils
+                .bech32
+                .encode_address(output.address.as_slice())?,
             amount: get_tx_output_coin_value(&output.amount),
             assets: self.collect_asset_records(&output.amount).into(),
         })
@@ -265,10 +261,13 @@ impl EventWriter {
     pub fn to_transaction_record(
         &self,
         body: &TransactionBody,
+        tx_hash: &str,
         aux_data: Option<&AuxiliaryData>,
     ) -> Result<TransactionRecord, Error> {
         let mut record = TransactionRecord::default();
 
+        record.hash.push_str(tx_hash);
+
         for component in body.iter() {
             match component {
                 TransactionBodyComponent::Fee(x) => {
@@ -349,7 +348,7 @@ impl EventWriter {
         Ok(record)
     }
 
-    pub fn to_block_record(&self, source: &Block, hash: &[u8]) -> Result<BlockRecord, Error> {
+    pub fn to_block_record(&self, source: &Block, hash: &Hash<32>) -> Result<BlockRecord, Error> {
         Ok(BlockRecord {
             body_size: source.header.header_body.block_body_size as usize,
             issuer_vkey: source.header.header_body.issuer_vkey.to_hex(),
@@ -357,19 +356,7 @@ impl EventWriter {
             hash: hex::encode(hash),
             number: source.header.header_body.block_number,
             slot: source.header.header_body.slot,
+            previous_hash: hex::encode(source.header.header_body.prev_hash),
         })
     }
 }
-
-#[cfg(test)]
-mod tests {
-    use super::ToBech32;
-
-    #[test]
-    fn beach32_encodes_ok() {
-        let bytes = hex::decode("01ec6ad5daee9febbe300c6160a36d4daf0c5266ae2fe8245cbb581390629814d8165fd547b6f3f6f55842a5f042bcb113e8e86627bc071f37").unwrap();
-        let bech32 = bytes.try_to_bech32("addr").unwrap();
-
-        assert_eq!(bech32, "addr1q8kx44w6a607h03sp3skpgmdfkhsc5nx4ch7sfzuhdvp8yrznq2ds9jl64rmdulk74vy9f0sg27tzylgapnz00q8rumsuhj834");
-    }
-}
