@@ -1,5 +1,9 @@
+pub mod stdout;
 pub mod terminal;
 
+#[cfg(feature = "logs")]
+pub mod logs;
+
 #[cfg(feature = "webhook")]
 pub mod webhook;
 
