@@ -1,7 +1,9 @@
+use std::sync::Arc;
 use std::time::Duration;
 
 use crate::pipelining::StageReceiver;
 use crate::utils::throttle::Throttle;
+use crate::utils::Utils;
 
 pub type Error = Box<dyn std::error::Error>;
 
@@ -11,7 +13,11 @@ use std::io::stdout;
 
 use super::format::*;
 
-pub fn reducer_loop(throttle_min_span: Duration, input: StageReceiver) -> Result<(), Error> {
+pub fn reducer_loop(
+    throttle_min_span: Duration,
+    input: StageReceiver,
+    utils: Arc<Utils>,
+) -> Result<(), Error> {
     let mut stdout = stdout();
 
     let mut throttle = Throttle::new(throttle_min_span);
@@ -23,6 +29,10 @@ pub fn reducer_loop(throttle_min_span: Duration, input: StageReceiver) -> Result
     loop {
         let (width, _) = crossterm::terminal::size()?;
         let evt = input.recv()?;
+
+        // notify progress to the pipeline
+        utils.track_sink_progress(&evt);
+
         throttle.wait_turn();
         let line = LogLine::new(evt, width as usize);
         stdout.execute(Print(line))?;
