@@ -9,9 +9,12 @@ use pallas::ouroboros::network::{
     multiplexer::Channel,
 };
 use serde::{de::Visitor, Deserializer};
-use serde_derive::{Deserialize, Serialize};
+use serde::{Deserialize, Serialize};
 
-use crate::{mapper::ChainWellKnownInfo, Error};
+use crate::{
+    utils::{ChainWellKnownInfo, Utils},
+    Error,
+};
 
 #[derive(Debug, Deserialize)]
 pub enum BearerKind {
@@ -37,10 +40,10 @@ impl FromStr for BearerKind {
 }
 
 /// A serialization-friendly chain Point struct using a hex-encoded hash
-#[derive(Debug, Serialize, Deserialize)]
+#[derive(Clone, Debug, Serialize, Deserialize)]
 pub struct PointArg(pub u64, pub String);
 
-impl TryInto<Point> for &PointArg {
+impl TryInto<Point> for PointArg {
     type Error = Error;
 
     fn try_into(self) -> Result<Point, Self::Error> {
@@ -49,8 +52,8 @@ impl TryInto<Point> for &PointArg {
     }
 }
 
-impl From<&Point> for PointArg {
-    fn from(other: &Point) -> Self {
+impl From<Point> for PointArg {
+    fn from(other: Point) -> Self {
         PointArg(other.0, hex::encode(&other.1))
     }
 }
@@ -70,6 +73,12 @@ impl FromStr for PointArg {
     }
 }
 
+impl ToString for PointArg {
+    fn to_string(&self) -> String {
+        format!("{},{}", self.0, self.1)
+    }
+}
+
 #[derive(Debug, Deserialize)]
 pub struct MagicArg(pub u64);
 
@@ -95,6 +104,12 @@ impl FromStr for MagicArg {
     }
 }
 
+impl Default for MagicArg {
+    fn default() -> Self {
+        Self(MAINNET_MAGIC)
+    }
+}
+
 pub(crate) fn deserialize_magic_arg<'de, D>(deserializer: D) -> Result<Option<MagicArg>, D::Error>
 where
     D: Deserializer<'de>,
@@ -137,7 +152,7 @@ where
 pub(crate) fn find_end_of_chain(
     channel: &mut Channel,
     well_known: &ChainWellKnownInfo,
-) -> Result<Point, crate::framework::Error> {
+) -> Result<Point, crate::Error> {
     let point = Point(
         well_known.shelley_known_slot,
         hex::decode(&well_known.shelley_known_hash)?,
@@ -151,3 +166,26 @@ pub(crate) fn find_end_of_chain(
         None => Err("failure acquiring end of chain".into()),
     }
 }
+
+pub(crate) fn define_start_point(
+    explicit_arg: &Option<PointArg>,
+    utils: &Utils,
+    cs_channel: &mut Channel,
+) -> Result<Point, Error> {
+    let cursor = utils.get_cursor_if_any();
+
+    match (cursor, explicit_arg) {
+        (Some(cursor), _) => {
+            log::info!("found persisted cursor, will use as starting point");
+            cursor.try_into()
+        }
+        (None, Some(arg)) => {
+            log::info!("explicit 'since' argument, will use as starting point");
+            arg.clone().try_into()
+        }
+        _ => {
+            log::info!("no starting point specified, will use tip of chain");
+            find_end_of_chain(cs_channel, &utils.well_known)
+        }
+    }
+}
