@@ -1,4 +1,5 @@
 use minicbor::data::Tag;
+use std::fmt::Debug;
 
 use log::{info, warn};
 
@@ -17,11 +18,7 @@ use pallas::{
 
 use std::sync::mpsc::{Receiver, SyncSender};
 
-use crate::{
-    framework::{Error, EventData},
-    mapper::EventWriter,
-    utils::SwallowResult,
-};
+use crate::{mapper::EventWriter, model::EventData, utils::SwallowResult, Error};
 
 #[derive(Debug)]
 pub struct Content(u32, Header);
@@ -50,14 +47,19 @@ impl DecodePayload for Content {
 
 impl BlockLike for Content {
     fn block_point(&self) -> Result<Point, Box<dyn std::error::Error>> {
-        let hash = crypto::hash_block_header(&self.1)?;
-        Ok(Point(self.1.header_body.slot, Vec::from(hash)))
+        let hash = crypto::hash_block_header(&self.1);
+        Ok(Point(self.1.header_body.slot, hash.to_vec()))
     }
 }
 
-#[derive(Debug)]
 struct Block2EventMapper(EventWriter);
 
+impl Debug for Block2EventMapper {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_tuple("Block2EventMapper").finish()
+    }
+}
+
 impl BlockObserver for Block2EventMapper {
     fn on_block_received(&self, body: Vec<u8>) -> Result<(), Error> {
         let maybe_block = alonzo::BlockWrapper::decode_fragment(&body[..]);
@@ -80,12 +82,18 @@ impl BlockObserver for Block2EventMapper {
     }
 }
 
-#[derive(Debug)]
 struct ChainObserver {
     block_requests: SyncSender<Point>,
     event_writer: EventWriter,
 }
 
+// workaround to put a stop on excessive debug requirement coming from Pallas
+impl Debug for ChainObserver {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_tuple("ChainObserver").finish()
+    }
+}
+
 impl Observer<Content> for ChainObserver {
     fn on_block(&self, cursor: &Option<Point>, _content: &Content) -> Result<(), Error> {
         info!("requesting block fetch for point {:?}", cursor);
